#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <GL/glut.h>
#include "cube.h"
#include "image_rgba.h"

int init();
void display();
void idle();
void reshape(int x, int y);
void keyboard(unsigned char key, int x, int y);
void mouse(int bn, int st, int x, int y);
void motion(int x, int y);
unsigned int load_texture_rgba(const char *fname);

float cam_theta, cam_phi = 25, cam_dist = 8;	// camera parameters
unsigned int alpha_tex, floor_tex;	// textures

int main(int argc, char **argv)
{
	glutInit(&argc, argv);
	glutInitWindowSize(800, 600);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutCreateWindow("test");

	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);
	glutMotionFunc(motion);

	if(init() == -1) {
		return 1;
	}

	glutMainLoop();
	return 0;
}


int init()
{
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);

	float amb_color[] = {0.35, 0.35, 0.35, 1.0};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, amb_color);

	// load transparent texture for the cube
	alpha_tex = load_texture_rgba("opengl_alpha.tga");
	if(!alpha_tex) {
		return -1;
	}

	// load floor texture
	floor_tex = load_texture_rgba("ground.tga");
	if(!floor_tex) {
		return -1;
	}

	return 0;
}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// setup view matrix
	glTranslatef(0, 0, -cam_dist);
	glRotatef(cam_phi, 1, 0, 0);
	glRotatef(cam_theta, 0, 1, 0);

	// set light position in world space (between model and view)
	float lpos[] = {-80, 120, 100, 1};
	glLightfv(GL_LIGHT0, GL_POSITION, lpos);

	glPushMatrix();
	glTranslatef(0, 1, 0);	// world matrix

	// ---- start drawing the scene ----

	// draw the transparent cube
	glBindTexture(GL_TEXTURE_2D, alpha_tex);
	glEnable(GL_TEXTURE_2D);

	draw_cube();
	glDisable(GL_BLEND);

	glPopMatrix();


	// also draw a floor
	/*
	glBindTexture(GL_TEXTURE_2D, floor_tex);
	glEnable(GL_TEXTURE_2D);

	glBegin(GL_QUADS);
	glTexCoord2f(0, 0);
	glVertex3f(-6, 0, 6);
	glTexCoord2f(8, 0);
	glVertex3f(6, 0, 6);
	glTexCoord2f(8, 8);
	glVertex3f(6, 0, -6);
	glTexCoord2f(0, 8);
	glVertex3f(-6, 0, -6);
	glEnd();

	glDisable(GL_TEXTURE_2D);
	*/

	glutSwapBuffers();
	assert(glGetError() == GL_NO_ERROR);
}

void idle()
{
	glutPostRedisplay();
}

void reshape(int x, int y)
{
	glViewport(0, 0, x, y);

	// if the window size changed, we need to calculate the projection matrix again
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(50.0, (float)x / (float)y, 0.5, 500.0);
}

void keyboard(unsigned char key, int x, int y)
{
	switch(key) {
	case 27:
	case 'q':
		exit(0);
	}
}

int bnstate[16];
int prev_x, prev_y;

void mouse(int bn, int st, int x, int y)
{
	prev_x = x;
	prev_y = y;
	bnstate[bn - GLUT_LEFT_BUTTON] = st == GLUT_DOWN;
}

void motion(int x, int y)
{
	int dx = x - prev_x;
	int dy = y - prev_y;
	prev_x = x;
	prev_y = y;

	if(bnstate[0]) {
		cam_theta += dx * 0.5;
		cam_phi += dy * 0.5;

		if(cam_phi < -90) cam_phi = -90;
		if(cam_phi > 90) cam_phi = 90;
	}
	if(bnstate[2]) {
		cam_dist += dy * 0.1;
		if(cam_dist < 0.0) cam_dist = 0.0;
	}
}

unsigned int load_texture_rgba(const char *fname)
{
	struct image_rgba *img = load_image_rgba(fname);
	if(!img) {
		return 0;
	}

	unsigned int tex;
	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, img->width, img->height, 0, GL_RGBA, GL_UNSIGNED_BYTE, img->pixels);
	free_image_rgba(img);

	return tex;
}
